@@ -45,7 +45,8 @@
 typedef enum {
 	NOT_RUNNING,
 	RUNNING,
-	MAKE_IT_STOP
+	MAKE_IT_STOP,
+	MAKE_IT_QUIT
 } RunLevel;
 
 enum {
@@ -57,6 +58,7 @@ enum {
 	COLUMN_TYPE,
 	COLUMN_READABLE_DATE,
 	COLUMN_DATE,
+	COLUMN_NO_FILES_FOUND,
 	NUM_COLUMNS
 };
 
@@ -65,12 +67,13 @@ const FindOptionTemplate templates[] = {
 	{ FIND_OPTION_TEXT, "'!' -name '%s'", N_("File name is not") }, 
 	{ FIND_OPTION_BOOL, "-mount", N_("Don't search mounted filesystems") },
 	{ FIND_OPTION_BOOL, "-follow", N_("Follow symbolic links") },	
-	{ FIND_OPTION_BOOL, "-depth", N_("Process directory contents depth first") },
+	{ FIND_OPTION_BOOL, "-depth", N_("Process folder contents depth first") },
 	{ FIND_OPTION_TEXT, "-user '%s'", N_("Owned by user") },
 	{ FIND_OPTION_TEXT, "-group '%s'", N_("Owned by group") },
 	{ FIND_OPTION_BOOL, "\\( -nouser -o -nogroup \\)", N_("Invalid user or group") },
 	{ FIND_OPTION_TIME, "-mtime '%s'", N_("Last time modified") },
 	{ FIND_OPTION_BOOL, "-size 0c \\( -type f -o -type d \\)", N_("File is empty") },
+	{ FIND_OPTION_TEXT, "-regex '%s'", N_("Matches regular expression") },
 	{ FIND_OPTION_END, NULL,NULL}
 };
 
@@ -88,10 +91,18 @@ static GtkWidget *start_dir_e = NULL;
 
 static GtkWidget *locate_entry = NULL;
 
+static GtkWidget *locate_path_entry = NULL;
+
+static GtkWidget *locate_browse_entry = NULL;
+
 static GtkWidget *nbook = NULL;
 
 static GtkWidget *app = NULL; 
 
+static GtkWidget *status_bar = NULL;
+
+static GtkWidget *progress_bar = NULL;
+
 static GtkWidget *file_selector = NULL;
 
 static GtkWidget *locate_results = NULL;
@@ -125,6 +136,7 @@ static gboolean lock = FALSE;
 static int current_template = 0;
 
 static char *filename = NULL;
+static gchar *locate_string = NULL;
 
 static GtkWidget *save_widget = NULL;
 
@@ -163,6 +175,8 @@ make_find_cmd (const char *start_dir)
 	GString *cmdbuf;
 	GList *list;
 	gchar *escape_dir = NULL;
+	gchar *regex = NULL;
+
 	
 	if (criteria_find==NULL) return NULL;
 
@@ -184,12 +198,19 @@ make_find_cmd (const char *start_dir)
 						  templates[opt->templ].option);
 				break;
 			case FIND_OPTION_TEXT:
-				s = quote_quote_string(opt->data.text);
-				g_string_append_printf(cmdbuf,
+				if (!strcmp(templates[opt->templ].option,"-regex '%s'"))
+					regex=quote_quote_string(opt->data.text);
+				else {
+					gchar *locale_s = NULL;
+					s = quote_quote_string(opt->data.text);
+					locale_s = g_locale_from_utf8(s, -1, NULL, NULL, NULL);
+					g_string_append_printf(cmdbuf,
 						  templates[opt->templ].option,
-						  s);
-				g_free(s);
-				g_string_append_c(cmdbuf, ' ');
+						  locale_s);
+					g_free(locale_s);
+					g_free(s);
+					g_string_append_c(cmdbuf, ' ');
+				}
 				break;
 			case FIND_OPTION_NUMBER:
 				g_string_append_printf(cmdbuf,
@@ -198,12 +219,17 @@ make_find_cmd (const char *start_dir)
 				g_string_append_c(cmdbuf, ' ');
 				break;
 			case FIND_OPTION_TIME:
-				s = quote_quote_string(opt->data.time);
-				g_string_append_printf(cmdbuf,
+				{
+					gchar *locale_s = NULL;
+					s = quote_quote_string(opt->data.time);
+					locale_s = g_locale_from_utf8(s, -1, NULL, NULL, NULL);
+					g_string_append_printf(cmdbuf,
 						  templates[opt->templ].option,
-						  s);
-				g_free(s);
-				g_string_append_c(cmdbuf, ' ');
+						  locale_s);
+					g_free(locale_s);
+					g_free(s);
+					g_string_append_c(cmdbuf, ' ');
+				}
 				break;
 			default:
 			        break;
@@ -212,6 +238,13 @@ make_find_cmd (const char *start_dir)
 	}
 	g_string_append (cmdbuf, "-print ");
 
+	if(regex!=NULL) 
+	{	
+		gchar *locale_regex = NULL;
+		locale_regex = g_locale_from_utf8 (regex, -1, NULL, NULL, NULL);
+		g_string_append_printf (cmdbuf, " |egrep '%s'", locale_regex);
+		g_free (locale_regex);
+	}
 	g_free(escape_dir);
 
 	return g_string_free(cmdbuf, FALSE);
@@ -221,8 +254,10 @@ static char *
 make_locate_cmd(void)
 {
 	GString *cmdbuf;
-	gchar *locate_string;
+	gchar *locate_path;
 	gchar *locate_command;
+	gchar *locale_locate_path;
+	gchar *locale_locate_string;
 	
 	locate_string = 
 		(gchar *)gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(locate_entry))));
@@ -231,17 +266,35 @@ make_locate_cmd(void)
 	if(!locate_string || !*locate_string) {
 		return NULL;
 	}
+	
+	if(locate_path_entry) {
+		locate_path = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(locate_path_entry), TRUE);
+		if(!locate_path) {
+			gnome_app_error (GNOME_APP(app),
+					 _("Start folder does not exist."));
+			return NULL;
+		}
+	} else
+		locate_path = NULL;
+
+	locale_locate_path = g_locale_from_utf8(locate_path, -1, NULL, NULL, NULL);
+	locale_locate_string = g_locale_from_utf8(locate_string, -1, NULL, NULL, NULL);
 
 	cmdbuf = g_string_new ("");
 	locate_command = g_find_program_in_path ("locate");
+
 	if (locate_command != NULL)
 	{
-		g_string_append_printf (cmdbuf, "%s '%s'", locate_command, locate_string);
+		g_string_append_printf (cmdbuf, "%s '%s*%s'", locate_command, locale_locate_path,
+					locale_locate_string);
 	} else {
-		g_string_append_printf (cmdbuf, "find \"/\" -name '%s' -mount", locate_string);
-	}
-	g_free (locate_string);
+		g_string_append_printf (cmdbuf, "find \"%s\" -name '%s' -mount", locale_locate_path, 
+					locale_locate_string);
+	}	
+	g_free (locate_path);
 	g_free (locate_command);
+	g_free (locale_locate_path);
+	g_free (locale_locate_string);
 	
 	return g_string_free (cmdbuf, FALSE);
 }
@@ -266,10 +319,11 @@ kill_after_nth_nl (GString *str, int n)
 static gchar *
 get_icon_of_file_with_mime_type (const gchar *file_name, const gchar *mime_type) 
 {
-	const char *icon_name = gnome_vfs_mime_get_icon(mime_type);
+	const char *icon_name; 
 	gchar *icon_path;
 	gchar *tmp;
 	
+	icon_name = gnome_vfs_mime_get_icon(mime_type);
 	if (g_file_test ((icon_name), G_FILE_TEST_EXISTS)) {
 		icon_path = g_strdup (icon_name);
 		return icon_path;
@@ -421,10 +475,11 @@ static gchar *
 get_readable_date (time_t file_time_raw)
 {
 	struct tm *file_time;
-	const char *format;
+	gchar *format;
 	GDate *today;
 	GDate *file_date;
 	guint32 file_date_age;
+	gchar *readable_date;
 
 	file_time = localtime (&file_time_raw);
 	file_date = g_date_new_dmy (file_time->tm_mday,
@@ -450,7 +505,10 @@ get_readable_date (time_t file_time_raw)
 		format = g_strdup(_("%m/%-d/%y, %-I:%M %p"));
 	}
 	
-	return strdup_strftime (format, file_time);
+	readable_date = strdup_strftime (format, file_time);
+	g_free (format);
+	
+	return readable_date;
 } 
 
 static gchar *
@@ -490,9 +548,9 @@ add_file_to_list_store(const gchar *file, GtkListStore *store, GtkTreeIter *iter
 	gchar *icon_path = get_icon_of_file_with_mime_type (file, mime_type);
 	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
 	GnomeVFSFileInfo *vfs_file_info = gnome_vfs_file_info_new ();
-	gchar *utf8_file = g_locale_to_utf8 (file, -1, NULL, NULL, NULL);
 	gchar *readable_size, *readable_date, *date;
 	gchar *utf8_base_name, *utf8_dir_name;
+	gchar *base_name, *dir_name;
 	
 	if (pixbuf != NULL) {
 		GdkPixbuf *scaled = gdk_pixbuf_scale_simple (pixbuf,
@@ -506,10 +564,12 @@ add_file_to_list_store(const gchar *file, GtkListStore *store, GtkTreeIter *iter
 	readable_size = gnome_vfs_format_file_size_for_display (vfs_file_info->size);
 	readable_date = get_readable_date (vfs_file_info->mtime);
 	date = get_basic_date (vfs_file_info->mtime);
-	utf8_base_name = g_locale_to_utf8 (g_path_get_basename(file), -1, NULL, 
-					   NULL, NULL);
-	utf8_dir_name = g_locale_to_utf8 (g_path_get_dirname(file), -1, NULL, 
-					  NULL, NULL);
+	
+	base_name = g_path_get_basename(file);
+	dir_name = g_path_get_dirname(file);
+	
+	utf8_base_name = g_locale_to_utf8 (base_name, -1, NULL, NULL, NULL);
+	utf8_dir_name = g_locale_to_utf8 (dir_name, -1, NULL, NULL, NULL);
 
 	gtk_list_store_append (GTK_LIST_STORE(store), iter); 
 	gtk_list_store_set (GTK_LIST_STORE(store), iter,
@@ -521,11 +581,13 @@ add_file_to_list_store(const gchar *file, GtkListStore *store, GtkTreeIter *iter
 			    COLUMN_TYPE, description,
 			    COLUMN_READABLE_DATE, readable_date,
 			    COLUMN_DATE, date,
+			    COLUMN_NO_FILES_FOUND, FALSE,
 			    -1);
 			    
-	gnome_vfs_file_info_clear (vfs_file_info);
+	gnome_vfs_file_info_unref (vfs_file_info);
 	g_object_unref (G_OBJECT (pixbuf));
-	g_free (utf8_file);
+	g_free (base_name);
+	g_free (dir_name);
 	g_free (utf8_base_name);
 	g_free (utf8_dir_name);
 	g_free (icon_path); 
@@ -592,19 +654,49 @@ add_atk_relation(GtkWidget *obj1, GtkWidget *obj2, AtkRelationType rel_type)
 
 
 static void
-really_run_command(char *cmd, char sepchar, RunLevel *running, GtkWidget *tree, GtkListStore *store, GtkTreeIter *iter)
+add_no_files_found_to_list_store(GtkListStore *store, GtkTreeIter *iter)
 {
-	int idle;
+	/* when the list is empty append a 'No Files Found.' message */
+	gtk_list_store_append (GTK_LIST_STORE(store), iter); 
+	gtk_list_store_set (GTK_LIST_STORE(store), iter,
+		    		COLUMN_ICON, NULL, 
+				COLUMN_NAME, _("No Files Found."),
+		    		COLUMN_PATH, "",
+			    	COLUMN_READABLE_SIZE, "",
+			    	COLUMN_SIZE, (gdouble) 0,
+			    	COLUMN_TYPE, "",
+		    		COLUMN_READABLE_DATE, "",
+		    		COLUMN_DATE, 0,
+				COLUMN_NO_FILES_FOUND, TRUE,
+		    		-1);
+}
+
+static gint
+update_progress_bar (gpointer p)
+{
+	gtk_progress_bar_pulse (GTK_PROGRESS_BAR (progress_bar));
+	
+	return TRUE;
+}
+
+static void
+really_run_command(char *cmd, char sepchar, gchar *utf8_pattern_str, RunLevel *running, GtkWidget *tree, GtkListStore *store, GtkTreeIter *iter)
+{
+	int timeout;
 	GString *string;
 	char ret[PIPE_READ_BUFFER];
 	int fd[2], fderr[2];
 	int i,n;
 	int pid;
+	int files_found;
+	char* str;
+	GPatternSpec *pattern;
 	GString *errors = NULL;
 	gboolean add_to_errors = TRUE;
 	
 	lock = TRUE;
 
+	pattern = g_pattern_spec_new (utf8_pattern_str);
 	/* reset scroll position and clear the tree view */
 	gtk_tree_view_scroll_to_point (GTK_TREE_VIEW(tree), 0, 0);
 	gtk_list_store_clear (GTK_LIST_STORE(store)); 
@@ -644,6 +736,7 @@ really_run_command(char *cmd, char sepchar, RunLevel *running, GtkWidget *tree,
 					_("Cannot perform the search.\n"
 					"The Bourne shell was not found\n"
 					"It is typically installed as /bin/sh or /usr/bin/sh."));
+				gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (app));
 				gtk_dialog_run (GTK_DIALOG (dialog));
 				gtk_widget_destroy (dialog);
 			}
@@ -655,8 +748,10 @@ really_run_command(char *cmd, char sepchar, RunLevel *running, GtkWidget *tree,
 	close(fd[1]);
 	close(fderr[1]);
 
-	idle = gtk_idle_add((GtkFunction)gtk_true,NULL);
-
+	gnome_appbar_pop (GNOME_APPBAR (status_bar));
+	gnome_appbar_push (GNOME_APPBAR (status_bar), _("Searching..."));
+	timeout = gtk_timeout_add (100, update_progress_bar, NULL);
+		
 	fcntl(fd[0], F_SETFL, O_NONBLOCK);
 	fcntl(fderr[0], F_SETFL, O_NONBLOCK);
 
@@ -668,8 +763,19 @@ really_run_command(char *cmd, char sepchar, RunLevel *running, GtkWidget *tree,
 			if (*running != RUNNING)
 				break;
 			if(ret[i] == sepchar) {
-				add_file_to_list_store(string->str, store, iter);
+				gchar *locale_base_name = NULL;
+				gchar *base_name = NULL;
+
+				base_name = g_path_get_basename (string->str);
+			        locale_base_name = g_locale_to_utf8 (base_name, -1, NULL, NULL, NULL);
+				
+				if (g_pattern_match_string (pattern, locale_base_name)) {
+				        add_file_to_list_store (string->str, store, iter);
+				}
+
 				g_string_assign (string, "");
+				g_free (locale_base_name);
+				g_free (base_name);
 			} else {
 				g_string_append_c (string, ret[i]);
 			}
@@ -681,12 +787,15 @@ really_run_command(char *cmd, char sepchar, RunLevel *running, GtkWidget *tree,
 		if (n > 0) {
 			ret[n] = '\0';
 			if (add_to_errors) {
-				if (errors == NULL)
-					errors = g_string_new (ret);
-				else
-					errors = g_string_append (errors, ret);
-				add_to_errors =
-					! kill_after_nth_nl (errors, 20);
+				if (strstr(ret, "ermission denied") == NULL) {
+					if (errors == NULL)
+						errors = g_string_new (ret);
+					else
+						errors = g_string_append (errors, ret);
+					
+					add_to_errors =
+						! kill_after_nth_nl (errors, 20);
+				}
 			}
 			fprintf (stderr, "%s", ret);
 		}
@@ -695,7 +804,7 @@ really_run_command(char *cmd, char sepchar, RunLevel *running, GtkWidget *tree,
 			break;
 		if(gtk_events_pending())
 			gtk_main_iteration_do(TRUE);
-		if (*running == MAKE_IT_STOP) {
+		if ((*running == MAKE_IT_STOP) || (*running == MAKE_IT_QUIT)) {
 			kill(pid, SIGKILL);
 			wait(NULL);
 		}
@@ -706,8 +815,19 @@ really_run_command(char *cmd, char sepchar, RunLevel *running, GtkWidget *tree,
 			if (*running != RUNNING) 
 				break;
 			if(ret[i] == sepchar) {
-				add_file_to_list_store(string->str, store, iter);
+				gchar *locale_base_name = NULL;
+				gchar *base_name = NULL;
+				
+				base_name = g_path_get_basename (string->str);
+			        locale_base_name = g_locale_to_utf8 (base_name, -1, NULL, NULL, NULL);
+				
+				if (g_pattern_match_string (pattern, locale_base_name)) {
+				        add_file_to_list_store (string->str, store, iter);
+				}
+				
 				g_string_assign (string, "");
+				g_free (locale_base_name);
+				g_free (base_name);
 			} else {
 				g_string_append_c (string, ret[i]);
 			}
@@ -718,21 +838,26 @@ really_run_command(char *cmd, char sepchar, RunLevel *running, GtkWidget *tree,
 	while((n = read(fderr[0], ret, PIPE_READ_BUFFER-1)) > 0) {
 		ret[n]='\0';
 		if (add_to_errors) {
-			if (errors == NULL)
-				errors = g_string_new (ret);
-			else
-				errors = g_string_append (errors, ret);
-			add_to_errors =
-				! kill_after_nth_nl (errors, 20);
+			if (strstr(ret, "ermission denied") == NULL) {			
+				if (errors == NULL)
+					errors = g_string_new (ret);
+				else
+					errors = g_string_append (errors, ret); 
+				
+				add_to_errors =
+						! kill_after_nth_nl (errors, 20);
+			}
 		}
 		fprintf (stderr, "%s", ret);
 	}
 
 	close(fd[0]);
 	close(fderr[0]);
-
-	gtk_idle_remove(idle);
 	
+	gtk_timeout_remove (timeout);
+	gnome_appbar_pop (GNOME_APPBAR (status_bar));
+	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress_bar), 0.0);
+		
 	/* if any errors occured */
 	if(errors) {
 		if ( ! add_to_errors) {
@@ -746,11 +871,36 @@ really_run_command(char *cmd, char sepchar, RunLevel *running, GtkWidget *tree,
 		g_string_free (errors, TRUE);
 	}
 
+	files_found = gtk_tree_model_iter_n_children (GTK_TREE_MODEL(store), NULL);
+
+	if (files_found == 0)
+		str = g_strdup (_("No files found"));
+	else
+		if (files_found == 1)
+			str = g_strdup (_("One file found"));
+		else
+			str = g_strdup_printf (_("%d files found"), files_found);
+	
+	gnome_appbar_push (GNOME_APPBAR (status_bar), str);
+
+	if (files_found == 0)
+		add_no_files_found_to_list_store (store, iter);
+	
+	g_free (str);
 	g_string_free (string, TRUE);
+	g_pattern_spec_free (pattern);
 
-	*running = NOT_RUNNING;
-
-	lock = FALSE;
+	if (*running == MAKE_IT_QUIT)
+	{
+	 	*running = NOT_RUNNING;
+		lock = FALSE;
+		gtk_main_quit ();
+ 	}
+ 	else
+ 	{
+		*running = NOT_RUNNING;
+		lock = FALSE;
+	}
 }
 
 static void
@@ -762,6 +912,18 @@ run_command(GtkWidget *w, gpointer data)
 	char *start_dir;
 
 	gtk_widget_set_sensitive(save_widget, FALSE);
+
+	if (buttons == find_buttons)
+	{
+		gtk_widget_set_sensitive (locate_buttons[0], FALSE);
+		gtk_widget_set_sensitive (locate_buttons[1], FALSE);
+	}
+	else
+	{
+		gtk_widget_set_sensitive (find_buttons[0], FALSE);
+		gtk_widget_set_sensitive (find_buttons[1], FALSE);
+	}
+	
 	if (buttons[0] == w) { /*we are in the stop button*/
 		if(find_running == RUNNING) {
 			find_running = MAKE_IT_STOP;
@@ -770,15 +932,16 @@ run_command(GtkWidget *w, gpointer data)
 			gtk_widget_hide(buttons[0]);
 			gtk_widget_show(buttons[1]);
 		}
-		return;
+
+		goto out;
 	}
 
 	if(start_dir_e) {
 		start_dir = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(start_dir_e), TRUE);
 		if(!start_dir) {
 			gnome_app_error (GNOME_APP(app),
-					 _("Start directory does not exist."));
-			return;
+					 _("Start folder does not exist."));
+			goto out;
 		}
 	} else
 		start_dir = NULL;
@@ -789,7 +952,7 @@ run_command(GtkWidget *w, gpointer data)
 	if (cmd == NULL) {
 		gnome_app_error (GNOME_APP(app),
 				 _("Cannot perform the search. Please specify a search criteria."));
-		return;
+		goto out;
 	}
 
 	if (!lock) {
@@ -798,7 +961,7 @@ run_command(GtkWidget *w, gpointer data)
 		gtk_widget_show(buttons[0]);
 		gtk_widget_hide(buttons[1]);
 
-		really_run_command(cmd, '\n', &find_running, find_tree, find_model, &find_iter);
+		really_run_command(cmd, '\n', "*", &find_running, find_tree, find_model, &find_iter);
 		
 		gtk_widget_set_sensitive(buttons[1], TRUE);
 		gtk_widget_hide(buttons[0]);
@@ -810,6 +973,19 @@ run_command(GtkWidget *w, gpointer data)
 	}
 	gtk_widget_set_sensitive(save_widget, TRUE);
 	g_free(cmd);
+
+out:
+	if (buttons == find_buttons)
+	{
+		gtk_widget_set_sensitive (locate_buttons[0], TRUE);
+		gtk_widget_set_sensitive (locate_buttons[1], TRUE);
+	}
+	else
+	{
+		gtk_widget_set_sensitive (find_buttons[0], TRUE);
+		gtk_widget_set_sensitive (find_buttons[1], TRUE);
+	}
+
 }
 
 static void
@@ -817,14 +993,16 @@ run_cmd_dialog(GtkWidget *wid, gpointer data)
 {
 	char *cmd;
 	char *start_dir;
+	gchar *locale_cmd = NULL;
 	GtkWidget *dlg;
 	GtkWidget *w, *label;
+	GtkWidget *vbox;
 
 	if(start_dir_e) {
 		start_dir = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(start_dir_e), TRUE);
 		if(!start_dir) {
 			gnome_app_error (GNOME_APP(app),
-					 _("Start directory does not exist"));
+					 _("Start folder does not exist"));
 			return;
 		}
 	} else
@@ -845,7 +1023,7 @@ run_cmd_dialog(GtkWidget *wid, gpointer data)
 	
 	dlg = gtk_dialog_new_with_buttons(_("Search Command Line"), GTK_WINDOW(app),
 			       GTK_DIALOG_DESTROY_WITH_PARENT,
-			       GTK_STOCK_OK, GTK_RESPONSE_OK,
+			       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 			       NULL);
 	
 	gtk_window_set_modal (GTK_WINDOW(dlg), TRUE);
@@ -855,18 +1033,26 @@ run_cmd_dialog(GtkWidget *wid, gpointer data)
 		      "allow_shrink", FALSE,
 		      "resizable", FALSE,
 		      NULL);
-		      
+	
+	vbox = gtk_vbox_new (FALSE, 4);
+    	g_object_set (G_OBJECT (vbox), "border_width", 6, NULL);
+	
 	gtk_dialog_set_default_response (GTK_DIALOG(dlg), GTK_RESPONSE_CLOSE); 
 
-	label = gtk_label_new(_("This is the command line that can be used to "
-			    "execute this search from the console."
-			    "\n\nCommand:"));
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), label, TRUE, TRUE, 0);
+	label = gtk_label_new_with_mnemonic (_("This is the command line that can be used to "
+			    "execute this search\nfrom the console."
+			    "\n\nC_ommand:"));
+	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
 	
 	w = gtk_entry_new();
 	gtk_editable_set_editable(GTK_EDITABLE(w), FALSE);
-	gtk_entry_set_text(GTK_ENTRY(w), cmd);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), w, FALSE, FALSE, 0);
+	locale_cmd = g_locale_to_utf8(cmd, -1, NULL, NULL, NULL);
+	gtk_entry_set_text(GTK_ENTRY(w), locale_cmd);
+	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
+
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), w);
+
+	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dlg)->vbox), vbox);
 
 	g_signal_connect_swapped (GTK_OBJECT (dlg), 
                              	  "response", 
@@ -882,6 +1068,7 @@ run_cmd_dialog(GtkWidget *wid, gpointer data)
 
 	gtk_widget_show_all(dlg); 
 	
+	g_free(locale_cmd);
 	g_free(cmd);
 }
 
@@ -946,8 +1133,10 @@ static gboolean
 launch_file (GtkWidget *w, GdkEventButton *event, gpointer data)
 {
 	gchar *file = NULL;
+	gchar *locale_file = NULL;
 	gchar *utf8_name = NULL;
 	gchar *utf8_path = NULL;
+	gboolean no_files_found = FALSE;
 	GtkListStore *store;
 	GtkTreeIter iter;
 	GtkTreeSelection *selection;
@@ -968,20 +1157,29 @@ launch_file (GtkWidget *w, GdkEventButton *event, gpointer data)
 		
 		gtk_tree_model_get(GTK_TREE_MODEL(store),&iter,COLUMN_NAME, &utf8_name,
 							       COLUMN_PATH, &utf8_path,
+							       COLUMN_NO_FILES_FOUND, &no_files_found,
 								-1);
-		file = g_build_filename (utf8_path, utf8_name, NULL);
+
+		if (!no_files_found) {
+			file = g_build_filename (utf8_path, utf8_name, NULL);
+			locale_file = g_locale_from_utf8(file, -1, NULL, NULL, NULL);
+			
+			if (nautilus_is_running ()) {
+				launch_nautilus (locale_file);
+			}
+			else {
+				if (!view_file_with_application(locale_file))	
+					gnome_error_dialog_parented(_("No viewer available for this mime type."),
+							    	    GTK_WINDOW(app));
+			}
+
+			g_free (file);
+			g_free (locale_file);
+		}
 		
-		if (nautilus_is_running ()) {
-			launch_nautilus (file);
-		}
-		else {
-			if (!view_file_with_application(file))	
-				gnome_error_dialog_parented(_("No viewer available for this mime type."),
-						    	    GTK_WINDOW(app));
-		}
-		g_free(utf8_name);
-		g_free(utf8_path);
-		g_free(file);
+		g_free (utf8_name);
+		g_free (utf8_path);
+
 		return TRUE;
 	}
 	return FALSE;
@@ -991,10 +1189,16 @@ static void
 quit_cb(GtkWidget *w, gpointer data)
 {
 	if(find_running == RUNNING)
-		find_running = MAKE_IT_STOP;
+	{
+		find_running = MAKE_IT_QUIT;
+		return;
+	}
 	if(locate_running == RUNNING)
-		locate_running = MAKE_IT_STOP;
-	gtk_main_quit();
+	{
+		locate_running = MAKE_IT_QUIT;
+		return;
+	}
+	gtk_main_quit ();
 }
 
 static void
@@ -1208,8 +1412,10 @@ create_find_page(void)
 	gtk_container_set_border_width(GTK_CONTAINER(vbox),GNOME_PAD_SMALL);
 
 	hbox = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
-	gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);
+	gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,GNOME_PAD_SMALL);
 	label = gtk_label_new_with_mnemonic(_("S_tarting in folder:"));
+	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	g_object_set (G_OBJECT (label), "xalign", 0.0, NULL);
 	gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
 	start_dir_e = gnome_file_entry_new("directory", _("Browse"));
 	gnome_file_entry_set_directory_entry(GNOME_FILE_ENTRY(start_dir_e), TRUE);
@@ -1223,9 +1429,11 @@ create_find_page(void)
 		add_atk_namedesc (GTK_WIDGET(w), _("Starting folder entry"), _("Enter the folder name where you want to start the search"));
 		add_atk_relation (gnome_entry, GTK_WIDGET(label), ATK_RELATION_LABELLED_BY); 
 	}
+	/* Clicking on 'OK' in the Browse dialog will start a search -- commenting out until someone can figure this out
 	g_signal_connect (G_OBJECT (gnome_entry), "activate",
 			  G_CALLBACK (advanced_search_activate),
 			  find_buttons);
+	*/
 
 	s = g_get_current_dir();
 	gtk_entry_set_text(GTK_ENTRY(w), s);
@@ -1296,7 +1504,7 @@ create_find_page(void)
 	image = gtk_image_new_from_stock("gtk-find", GTK_ICON_SIZE_BUTTON);
 	gtk_box_pack_start (GTK_BOX(hbox2), image, FALSE, FALSE, 0);
 	
-	label = gtk_label_new_with_mnemonic(_("F_ind"));
+	label = gtk_label_new_with_mnemonic(_("_Find"));
 	gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
 	
 	find_buttons[0] = gtk_button_new_from_stock(GTK_STOCK_STOP);
@@ -1318,9 +1526,12 @@ create_find_page(void)
              
 	/* create search results section */
 	frame = gtk_frame_new(NULL);
+	label = gtk_label_new_with_mnemonic (_("S_earch Results"));
+	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	g_object_set (G_OBJECT (label), "xalign", 0.0, NULL);
+	gtk_frame_set_label_widget (GTK_FRAME (frame), label);
 	gtk_paned_pack2 (GTK_PANED (vpaned), frame, TRUE, FALSE);
 	gtk_widget_set_size_request (frame, -1, 90);
-	gtk_frame_set_label ( GTK_FRAME(frame), _("Results"));
 	find_results = gtk_scrolled_window_new(NULL,NULL);
 	gtk_widget_set_usize(find_results,600,90);
 	gtk_widget_set_sensitive(find_results, FALSE);
@@ -1336,7 +1547,8 @@ create_find_page(void)
 					 G_TYPE_DOUBLE,
 					 G_TYPE_STRING,
 					 G_TYPE_STRING,
-					 G_TYPE_STRING);				 
+					 G_TYPE_STRING,
+					 G_TYPE_BOOLEAN);				 
 	
 	find_tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(find_model));
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (find_tree), TRUE);
@@ -1347,6 +1559,8 @@ create_find_page(void)
 	g_signal_connect(G_OBJECT(find_tree), "button_press_event",
 		         G_CALLBACK(launch_file),NULL);		   
 
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), find_tree);
+
 	gtk_container_add(GTK_CONTAINER(find_results),GTK_WIDGET(find_tree));
 	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(find_results));
 	
@@ -1428,6 +1642,18 @@ run_locate_command(GtkWidget *w, gpointer data)
 	GtkWidget **buttons = data;
 
         gtk_widget_set_sensitive(save_widget, FALSE);
+
+	if (buttons == find_buttons)
+	{
+		gtk_widget_set_sensitive (locate_buttons[0], FALSE);
+		gtk_widget_set_sensitive (locate_buttons[1], FALSE);
+	}
+	else
+	{
+		gtk_widget_set_sensitive (find_buttons[0], FALSE);
+		gtk_widget_set_sensitive (find_buttons[1], FALSE);
+	}
+
 	if (buttons[0] == w) { /*we are in the stop button*/
 		if(locate_running == RUNNING)
 			locate_running = MAKE_IT_STOP;
@@ -1435,7 +1661,7 @@ run_locate_command(GtkWidget *w, gpointer data)
 			gtk_widget_show(buttons[1]);
 			gtk_widget_set_sensitive(buttons[1], FALSE);
 			gtk_widget_set_sensitive(save_widget, TRUE);
-		return;
+		goto out;
 	}
 
 	cmd = make_locate_cmd();
@@ -1444,7 +1670,7 @@ run_locate_command(GtkWidget *w, gpointer data)
 		gnome_app_error (GNOME_APP(app),
 				 _("Cannot perform the search. "
 				   "Please specify a search criteria."));
-		return;
+		goto out;
 	}
 	
 	if (!lock) {	
@@ -1454,7 +1680,7 @@ run_locate_command(GtkWidget *w, gpointer data)
 		gtk_widget_set_sensitive(buttons[1], FALSE);
 		gtk_widget_set_sensitive(locate_results, TRUE);
 
-		really_run_command(cmd, '\n', &locate_running, locate_tree, locate_model, &locate_iter);
+		really_run_command(cmd, '\n', locate_string, &locate_running, locate_tree, locate_model, &locate_iter);
 			
 		gtk_widget_hide(buttons[0]);
 		gtk_widget_set_sensitive(buttons[0], FALSE);
@@ -1466,8 +1692,24 @@ run_locate_command(GtkWidget *w, gpointer data)
 				_("A search is already running.  Please wait for the search "
 				  "to complete or cancel it."));
 	}
+	
 	gtk_widget_set_sensitive(save_widget, TRUE);
+	
 	g_free(cmd);
+
+out:
+	if (buttons == find_buttons)
+	{
+		gtk_widget_set_sensitive (locate_buttons[0], TRUE);
+		gtk_widget_set_sensitive (locate_buttons[1], TRUE);
+	}
+	else
+	{
+		gtk_widget_set_sensitive (find_buttons[0], TRUE);
+		gtk_widget_set_sensitive (find_buttons[1], TRUE);
+	}
+
+
 }
 
 static void
@@ -1507,24 +1749,34 @@ stop_search (GtkWidget *w, GdkEventKey *event)
 static GtkWidget *
 create_locate_page(void)
 {
-	GtkWidget *w, *vbox, *hbox, *hbox2, *entry;
+	GtkWidget *w, *vbox, *hbox, *hbox2, *entry, *table;
 	GtkWidget *image, *label, *frame, *align;
 	GtkTreeViewColumn *column;	
 	static gchar *history = NULL;
 	GtkCellRenderer *renderer;
+	gchar *s;
 
 	vbox = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), GNOME_PAD_SMALL);
 
 	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, GNOME_PAD_SMALL);
+	
+	table = gtk_table_new (2, 2, FALSE);
+	gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
+	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
+	gtk_container_add (GTK_CONTAINER (hbox), table);
+	
 	w = gtk_label_new_with_mnemonic(_("Find files _named:"));
-	gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
+	gtk_label_set_justify (GTK_LABEL (w), GTK_JUSTIFY_LEFT);
+	g_object_set (G_OBJECT (w), "xalign", 0.0, NULL);
+	
+	gtk_table_attach(GTK_TABLE(table), w, 0, 1, 0, 1, GTK_FILL, 0, 0, 1);
 
 	locate_entry = gnome_entry_new(history);
 	gtk_label_set_mnemonic_widget(GTK_LABEL(w), gnome_entry_gtk_entry(GNOME_ENTRY(locate_entry)));
 	gnome_entry_set_max_saved(GNOME_ENTRY(locate_entry), 10);
-	gtk_box_pack_start(GTK_BOX(hbox), locate_entry, TRUE, TRUE, 0);
+	gtk_table_attach(GTK_TABLE(table), locate_entry, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 	entry =  gnome_entry_gtk_entry (GNOME_ENTRY(locate_entry));
        
 	if (GTK_IS_ACCESSIBLE (gtk_widget_get_accessible(locate_entry)))
@@ -1539,6 +1791,35 @@ create_locate_page(void)
 			  G_CALLBACK (locate_activate),
 			  locate_buttons);
 			  
+	label = gtk_label_new_with_mnemonic(_("S_tarting in folder:"));
+	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	g_object_set (G_OBJECT (label), "xalign", 0.0, NULL);
+	
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
+	
+	locate_path_entry = gnome_file_entry_new("directory", _("Browse"));
+	gnome_file_entry_set_directory_entry(GNOME_FILE_ENTRY(locate_path_entry), TRUE);
+	w = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY(locate_path_entry));
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), w);
+	locate_browse_entry = gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY(locate_path_entry));
+	gtk_table_attach(GTK_TABLE(table), locate_path_entry, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+	
+	if (gail_loaded)
+	{ 
+		add_atk_namedesc (GTK_WIDGET(locate_browse_entry), _("Starting folder entry"), _("Enter the folder name where you want to start the search"));
+		add_atk_namedesc (GTK_WIDGET(w), _("Starting folder entry"), _("Enter the folder name where you want to start the search"));
+		add_atk_relation (locate_browse_entry, GTK_WIDGET(label), ATK_RELATION_LABELLED_BY); 
+	}
+	/* Clicking on 'OK' in the Browse dialog will start a search -- commenting out until someone can figure this out
+	g_signal_connect (G_OBJECT (locate_browse_entry), "activate",
+			  G_CALLBACK (locate_activate),
+			  locate_buttons);
+	*/
+			  
+	s = g_get_current_dir();
+	gtk_entry_set_text(GTK_ENTRY(w), s);
+	g_free(s);
+	
 	hbox = gtk_hbutton_box_new();
 	gtk_button_box_set_layout (GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
 	gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,0);
@@ -1556,7 +1837,7 @@ create_locate_page(void)
 	image = gtk_image_new_from_stock("gtk-find", GTK_ICON_SIZE_BUTTON);
 	gtk_box_pack_start (GTK_BOX(hbox2), image, FALSE, FALSE, 0);
 	
-	label = gtk_label_new_with_mnemonic(_("F_ind"));
+	label = gtk_label_new_with_mnemonic(_("_Find"));
 	gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
 
 	locate_buttons[0] = gtk_button_new_from_stock(GTK_STOCK_STOP);
@@ -1577,8 +1858,11 @@ create_locate_page(void)
 
 	/* create search results section */
 	frame = gtk_frame_new(NULL);
+	label = gtk_label_new_with_mnemonic (_("S_earch Results"));
+	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	g_object_set (G_OBJECT (label), "xalign", 0.0, NULL);
+	gtk_frame_set_label_widget (GTK_FRAME (frame), label);
 	gtk_box_pack_start(GTK_BOX(vbox),frame,TRUE,TRUE,0);
-	gtk_frame_set_label ( GTK_FRAME(frame), _("Results"));
 	locate_results = gtk_scrolled_window_new(NULL,NULL);
 	gtk_widget_set_sensitive(GTK_WIDGET(locate_results), FALSE); 
 	gtk_container_set_border_width(GTK_CONTAINER(locate_results), GNOME_PAD_SMALL);
@@ -1595,7 +1879,8 @@ create_locate_page(void)
 					  G_TYPE_DOUBLE,
 					  G_TYPE_STRING,
 					  G_TYPE_STRING,
-					  G_TYPE_STRING);
+					  G_TYPE_STRING,
+					  G_TYPE_BOOLEAN);
 	
 	locate_tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(locate_model));
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (locate_tree), TRUE);
@@ -1605,6 +1890,8 @@ create_locate_page(void)
 	
 	g_signal_connect(G_OBJECT(locate_tree), "button_press_event",
 		         G_CALLBACK(launch_file),NULL);		   
+
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), locate_tree);
 	
 	gtk_container_add(GTK_CONTAINER(locate_results),GTK_WIDGET(locate_tree));
 	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(locate_results));
@@ -1688,9 +1975,9 @@ create_window(void)
 	gtk_container_set_border_width(GTK_CONTAINER(nbook),GNOME_PAD_SMALL);
 
 	gtk_notebook_append_page(GTK_NOTEBOOK(nbook),create_locate_page(),
-				 gtk_label_new_with_mnemonic(_("Si_mple")));  /* Can we connect to this  */
+				 gtk_label_new (_("Simple")));  /* Can we connect to this  */
 	gtk_notebook_append_page(GTK_NOTEBOOK(nbook),create_find_page(),
-				 gtk_label_new_with_mnemonic(_("Ad_vanced")));
+				 gtk_label_new (_("Advanced")));
 
 	return nbook;
 }
@@ -1729,7 +2016,7 @@ about_cb (GtkWidget *widget, gpointer data)
 	g_free (file);
 
 	about = gnome_about_new(_("GNOME Search Tool"), VERSION,
-				_("(C) 1998,2000 the Free Software Foundation"),
+				_("(C) 1998,2002 the Free Software Foundation"),
 				_("Frontend to the unix find/locate "
 				  "commands"),
 				authors,
@@ -1740,6 +2027,7 @@ about_cb (GtkWidget *widget, gpointer data)
 		gdk_pixbuf_unref (pixbuf);
 	}
 										
+	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (data));
 	g_signal_connect (G_OBJECT (about), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed), &about);
 	gtk_widget_show (about);
@@ -1771,7 +2059,7 @@ save_results(GtkFileSelection *selector, gpointer user_data)
 			 GTK_BUTTONS_YES_NO,
 			 _("File %s already exists. Overwrite?"),
 			 filename);
-			 
+		gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW (app));
 		response = gtk_dialog_run (GTK_DIALOG (dialog));
 		
 		gtk_widget_destroy (GTK_WIDGET(dialog));
@@ -1863,26 +2151,33 @@ save_session (GnomeClient *client, gint phase,
 static gint
 die (GnomeClient *client, gpointer client_data)
 {
-	gtk_main_quit ();
+	quit_cb (NULL, NULL);
+	return TRUE;
 }
 
-static GnomeUIInfo file_menu[] = {
-	GNOMEUIINFO_ITEM_STOCK(N_("S_how Command"), "", run_cmd_dialog,GTK_STOCK_NEW),
-	GNOMEUIINFO_ITEM_STOCK(N_("Save Results _As..."), "",show_file_selector,GTK_STOCK_SAVE_AS),
+static GnomeUIInfo search_menu[] = {
+	GNOMEUIINFO_ITEM_STOCK (N_("S_how Command..."), 
+				N_("Show the command used to perform the search"), 
+				run_cmd_dialog,
+				GTK_STOCK_NEW),
+	GNOMEUIINFO_ITEM_STOCK (N_("Save Results _As..."), 
+			        N_("Save the search results"), 
+				show_file_selector,
+				GTK_STOCK_SAVE_AS),
 	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_EXIT_ITEM(quit_cb,NULL),
+	GNOMEUIINFO_MENU_QUIT_ITEM (quit_cb,NULL),
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo help_menu[] = {  
-	GNOMEUIINFO_HELP("gnome-search-tool"),
-	GNOMEUIINFO_MENU_ABOUT_ITEM(about_cb,NULL),
+	GNOMEUIINFO_HELP ("gnome-search-tool"),
+	GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, NULL),
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo gsearch_menu[] = {
-	GNOMEUIINFO_MENU_FILE_TREE(file_menu),
-	GNOMEUIINFO_MENU_HELP_TREE(help_menu),
+	GNOMEUIINFO_SUBTREE (N_("Sear_ch"), search_menu),
+	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
         GNOMEUIINFO_END
 };
 
@@ -1925,10 +2220,20 @@ main(int argc, char *argv[])
 	g_signal_connect (client, "save_yourself",
 			  G_CALLBACK (save_session), (gpointer)argv[0]);
 
-	g_signal_connect (client, "die", G_CALLBACK (die), NULL);		 
-			 
+	g_signal_connect (client, "die", G_CALLBACK (die), NULL);
+		
+	/* set up status bar */
+	status_bar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
+	gnome_app_set_statusbar (GNOME_APP (app), status_bar);
+	progress_bar = GTK_WIDGET (gnome_appbar_get_progress (
+				GNOME_APPBAR (status_bar)));
+	g_object_set (G_OBJECT (progress_bar), 
+		      "pulse_step", 0.1,
+		      NULL);
+				      
 	/*set up the menu*/
-        gnome_app_create_menus(GNOME_APP(app), gsearch_menu);
+        gnome_app_create_menus_with_data (GNOME_APP (app), gsearch_menu, app);
+	gnome_app_install_menu_hints (GNOME_APP (app), gsearch_menu);
 
 	search = create_window();
 	gtk_widget_show_all(search);
@@ -1937,7 +2242,7 @@ main(int argc, char *argv[])
 
 	gnome_app_set_contents(GNOME_APP(app), search);
 	
-      	hints.min_width = 320;
+      	hints.min_width = 460;
       	hints.min_height = 360;
 	gtk_window_set_geometry_hints(GTK_WINDOW(app), GTK_WIDGET(app),
 				      &hints, GDK_HINT_MIN_SIZE);
@@ -1945,7 +2250,7 @@ main(int argc, char *argv[])
 	gtk_widget_show(app);
 
 	gtk_window_set_focus(GTK_WINDOW(app), GTK_WIDGET(gnome_entry_gtk_entry(GNOME_ENTRY(locate_entry))));
-        save_widget=file_menu[1].widget;
+        save_widget = search_menu[1].widget;
 
 	gtk_main ();
 
